"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.customFromCursorInner = void 0;
const internal_1 = require("@fluidframework/runtime-utils/internal");
const internal_2 = require("@fluidframework/core-utils/internal");
const index_js_1 = require("../../core/index.js");
const index_js_2 = require("../../util/index.js");
const index_js_3 = require("../core/index.js");
const leafNodeSchema_js_1 = require("../leafNodeSchema.js");
const objectNodeTypes_js_1 = require("../objectNodeTypes.js");
/**
 * Builds an {@link CustomTree} from a cursor in Nodes mode.
 */
function customFromCursorInner(reader, options, schema, childHandler) {
    const type = reader.type;
    const nodeSchema = schema.get(type) ?? (0, index_js_2.fail)("missing schema for type in cursor");
    switch (type) {
        case leafNodeSchema_js_1.numberSchema.identifier:
        case leafNodeSchema_js_1.booleanSchema.identifier:
        case leafNodeSchema_js_1.nullSchema.identifier:
        case leafNodeSchema_js_1.stringSchema.identifier:
            (0, internal_2.assert)(reader.value !== undefined, 0xa50 /* out of schema: missing value */);
            (0, internal_2.assert)(!(0, internal_1.isFluidHandle)(reader.value), 0xa51 /* out of schema: unexpected FluidHandle */);
            return reader.value;
        case leafNodeSchema_js_1.handleSchema.identifier:
            (0, internal_2.assert)(reader.value !== undefined, 0xa52 /* out of schema: missing value */);
            (0, internal_2.assert)((0, internal_1.isFluidHandle)(reader.value), 0xa53 /* out of schema: expected FluidHandle */);
            return options.valueConverter(reader.value);
        default: {
            (0, internal_2.assert)(reader.value === undefined, 0xa54 /* out of schema: unexpected value */);
            if (nodeSchema.kind === index_js_3.NodeKind.Array) {
                const fields = (0, index_js_1.inCursorField)(reader, index_js_1.EmptyKey, () => (0, index_js_1.mapCursorField)(reader, () => childHandler(reader, options, schema)));
                return fields;
            }
            else {
                const fields = {};
                (0, index_js_1.forEachField)(reader, () => {
                    const children = (0, index_js_1.mapCursorField)(reader, () => childHandler(reader, options, schema));
                    if (children.length === 1) {
                        const storedKey = reader.getFieldKey();
                        const key = (0, objectNodeTypes_js_1.isObjectNodeSchema)(nodeSchema) && !options.useStoredKeys
                            ? (nodeSchema.storedKeyToPropertyKey.get(storedKey) ??
                                (0, index_js_2.fail)("missing property key"))
                            : storedKey;
                        // Length is checked above.
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        fields[key] = children[0];
                    }
                    else {
                        (0, internal_2.assert)(children.length === 0, 0xa19 /* invalid children number */);
                    }
                });
                return fields;
            }
        }
    }
}
exports.customFromCursorInner = customFromCursorInner;
//# sourceMappingURL=customTree.js.map